local revision = tonumber(string.sub("$Revision: 50 $", 12, -3))
local rss = _G.RShamanStat
if rss.revision < revision then rss.revision = revision end

-- "banker's rounding" function to mirror Blizzard's calculations (round 0.5 to even)
function rss.bankRound(x)
	if x == 0 then return 0; end
	local signX = x/math.abs(x);
	local answer = signX * math.floor(math.abs(x + 0.5 * signX));
	if x - math.floor(x) == 0.5 then
		if answer/2 ~= math.floor(answer/2) then
			answer = answer - signX
		end
	end
	return answer;
end

-- determine the number of ticks and tick interval (in seconds) for a HoT or totem, given the player's current Spell Haste
-- from http://www.totemspot.com/vb/entry.php?b=55
function rss.CalcTicks(spellID)
	return rss.CalcTicksFromHaste(spellID, rss.playerState.spellHaste);
end

function rss.CalcTicksFromHaste(spellID, haste)
	local tickCount = 0;
	local tickInterval = 0;
	if rss.SPELLS[spellID] and rss.SPELLS[spellID].ticks then
		local ticks = rss.SPELLS[spellID].ticks;
		tickInterval = ticks.tick/(1 + haste);
		if rss.SPELLS[spellID].totem then
			tickCount = 1 + math.floor(ticks.duration/tickInterval);
		else
			tickCount = rss.bankRound(ticks.duration/tickInterval);
		end
	end
	return tickCount, tickInterval;
end

-- determine the spell haste% required to receive one more tick than current number
-- from http://www.totemspot.com/vb/entry.php?b=55
function rss.CalcBreakpoint(spellID)
	if not rss.SPELLS[spellID] or not rss.SPELLS[spellID].ticks then return nil; end
	local ticks = rss.SPELLS[spellID].ticks;
	local currentTicks = rss.CalcTicks(spellID);
	if currentTicks < 1 then return; end
	local newTicks = math.floor(currentTicks + 1);
	return rss.CalcHaste(spellID, newTicks);
end

function rss.CalcHaste(spellID, tickCount)
	if not rss.SPELLS[spellID] or not rss.SPELLS[spellID].ticks then return nil; end
	local ticks = rss.SPELLS[spellID].ticks;
	local tickSpeed;
	if rss.SPELLS[spellID].totem then
		tickSpeed = (1000*ticks.duration)/(tickCount - 1); -- milliseconds
	else
		tickSpeed = (1000*ticks.duration)/(tickCount - 0.5); -- milliseconds
		if 2*math.floor(tickCount/2) == tickCount then
			tickSpeed = math.floor(tickSpeed) + 0.5;
		else
			tickSpeed = math.ceil(tickSpeed) - 0.5;
		end
	end
	return ((1000*ticks.tick)/tickSpeed - 1.0);
end

function rss.PrintBreakpoints()
	for spellID, spell in pairs(rss.SPELLS) do
		if spell.ticks and not spell.noHaste and not spell.alias then
			local spellName = GetSpellInfo(spellID);
			if spellName then
				local buffedBreakpoint = 100*rss.CalcBreakpoint(spellID);
				local unbuffedBreakpoint = 100*((1+buffedBreakpoint/100)/1.05 - 1);
				print(("%s : %d ticks\nNeed %.2f%% unbuffed Haste or %.2f%% buffed Haste for additional tick"):format(spellName, rss.CalcTicks(spellID), unbuffedBreakpoint, buffedBreakpoint));
			end
		end
	end
end